/*
*  Constrained Variable-Size Simplex Algorithm Example
*  R. Horst in "Nichtlineare Optimierung"
*  (Kurs 0854, Fernuniversitt Hagen)
*
*  version 20.4.2002
*  (c) 2002 Ingo D. Rullhusen <d01c@uni-bremen.de>
* 
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
* 
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <libvssa.h>


typedef struct
{
   double x1, x2;
} VECT;


double squ( double x ){ return(x*x); }


double f( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( 2.0*squ(v->x1) + 2.0*v->x1*v->x2 + squ(v->x2)
           - 10.0*v->x1 - 10.0*v->x2 );
}


double g1( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( squ(v->x1) + squ(v->x2) - 5.0 );
}


double g2( double *x, void *data )
{
   VECT *v = (VECT*) x;
   (*((int*)data))++;
   return( 3.0*v->x1 + v->x2 - 6.0 );
}



int main()
{
   int major, minor, sub, i, fi, gi[4];
   VECT min, max, *v, *dv;
   MIN_PROBLEM_C mprob;
   MIN_SOLUTION_C *solution;

   LibVSSAversion(&major,&minor,&sub);
   printf("\nusing libvssa Version %i.%i.%i\n",major,minor,sub);

   mprob.n = 2;

   mprob.x_min = (double*) &min;
   mprob.x_max = (double*) &max;

   min.x1 = 0.0;
   min.x2 = 0.0;

   max.x1 = min.x1 + 0.1;
   max.x2 = min.x2 + 0.1;

   mprob.func_data = (void*)&fi;
   fi = 0;
   mprob.func = f;

   mprob.m = 2;
   mprob.ief = calloc( mprob.m, sizeof( double(*)( double*, void* ) ) );
   mprob.ief[0] = g1;
   mprob.ief[1] = g2;
   mprob.ief_data = calloc( mprob.m, sizeof(void*) );
   for ( i=0; i<mprob.m; i++ )
   {
      mprob.ief_data[i] = &gi[i];
      gi[i] = 0;
   }

   mprob.k = 0;
   mprob.eqf = NULL;

   mprob.maxitno_vssa = 5000;
   mprob.maxitno_penalty = 50;
   mprob.eps = 1e-14;
   
   mprob.alpha = 1.0;
   mprob.beta = 0.5;
   mprob.gamma = 2.0;

   mprob.r0 = 1e3;
   mprob.rf = 10.0;

   solution = CreateSolutionC( &mprob );

   printf("\nFernuniversitt Hagen Kurs 0854 Beispiel 2.13\n");
   printf(" no of function calls: %i\n",*(int*)mprob.func_data);
   printf(" termination code: %i no of iterations: %i\n",solution->term,solution->itnop);
   printf(" f=%f df=%f\n",solution->y,solution->dy);

   v  = (VECT*) solution->x;
   dv = (VECT*) solution->dx;
   printf(" x1=%f dx=%f\n",v->x1,dv->x1);
   printf(" x2=%f dx=%f\n",v->x2,dv->x2);

   printf("\n");
   printf(" data from %i inequation constraints\n",solution->m);
   for ( i=0; i<solution->m; i++ )
   {
      printf(" g%i=%f Lagrange=%f Lerror=%f\n",i+1,solution->dief[i],solution->ui[i],solution->uierr[i]);
   }
   printf("\n");

   DeleteSolutionC( &solution );

   return(0);
}
